/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <Eina.h>
#include "contacts_app.h"
#include "data/data_common.h"

#define YEAR_BASE 1900
#define YEAR_FACTOR 10000
#define MONTH_BASE 1
#define MONTH_FACTOR 100
#define DATE_STRING_LENGTH 11

static const char *_fields[] = {"First Name", "Last Name", "Phone Number", "Email", "Url", "Company", "Birthday", "Note"};

static contacts_list_h _run_query(const char *table_uri, contacts_filter_h filter);
static bool _create_filter(const char *uri, contacts_filter_h *filter);
static contacts_filter_h _create_int_filter(const char *table_uri, unsigned int filter_field, int int_val);
static contacts_filter_h _create_str_filter(const char *table_uri, unsigned int filter_field, const char *str_val);
static contacts_filter_h _create_bool_filter(const char *table_uri, unsigned int filter_field, bool bool_val);

/**
 * @brief Function connects to the contacts database.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_connect(void)
{
	int ret = contacts_connect();
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_connect() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function disconnects from the contacts database.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_disconnect(void)
{
	int ret = contacts_disconnect();
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_disconnect() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function creates new database record described by provided URI.
 * @param[in] uri The URI of database view.
 * @param[out] rec_h The record's handle created.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_create_record(const char *uri, contacts_record_h *rec_h)
{
	int ret = contacts_record_create(uri, rec_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_create() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function inserts record to the database.
 * @param[in] rec_h The handle of the record to be inserted.
 * @param[out] id The identifier of the inserted record.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_insert_record(contacts_record_h rec_h, int *id)
{
	int ret = contacts_db_insert_record(rec_h, id);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_insert_record() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function updates the record in a database.
 * @param[in] rec_h The handle of the record to be updated.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_update_record(contacts_record_h rec_h)
{
	int ret = contacts_db_update_record(rec_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_update_record() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function obtains the record from database.
 * @param[in] uri The URI of database view.
 * @param[in] id The identifier of the record field.
 * @param[in] rec_h The handle of the obtained record.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_get_record(const char *uri, int id, contacts_record_h *rec_h)
{
	int ret = contacts_db_get_record(uri, id, rec_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_record() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function obtains all the records from database.
 * @param[in] uri The URI of database view.
 * @param[in] list_h The handle of the list where obtained records are stored.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_get_all_records(const char *uri, contacts_list_h *list_h)
{
	int ret = contacts_db_get_all_records(uri, 0, 0, list_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_all_records() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function obtains the child of parent record from database.
 * @param[in] parent_h The handle of the parent record.
 * @param[in] parent_id The identifier of the parent record field.
 * @param[out] rec_h The handle of the obtained child record.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_get_record_child(contacts_record_h parent_h, int parent_id, contacts_record_h *rec_h)
{
	int ret = contacts_record_get_child_record_at_p(parent_h, parent_id, 0, rec_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_child_record_at_p() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function adds a child to the parent record in database.
 * @param[in] parent_h The handle of the parent record.
 * @param[in] uri The URI of database view.
 * @param[in] parent_id The identifier of the parent record field.
 * @param[out] rec_h The handle of the child record newly created.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_add_record_child(contacts_record_h parent_h, const char *uri, int parent_id, contacts_record_h *rec_h)
{
	int ret;

	ret = contacts_record_create(uri, rec_h);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_create() failed. Err= %d.", ret);
		return false;
	}

	ret = contacts_record_add_child_record(parent_h, parent_id, *rec_h);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_add_child_record() failed. Err= %d.", ret);
		contacts_record_destroy(*rec_h, true);
		*rec_h = NULL;
	}

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function gets a boolean value from the given record field.
 * @param[in] rec_h The handle of the record whose boolean field value is to be obtained.
 * @param[in] id The identifier of the record's field.
 * @param[out] value The value obtained from given record's field.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_get_bool(contacts_record_h rec_h, int id, bool *value)
{
	int ret = contacts_record_get_bool(rec_h, id, value);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_bool() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function gets a integer value from the given record field.
 * @param[in] rec_h The handle of the record whose integer field value is to be obtained.
 * @param[in] id The identifier of the record's field.
 * @param[out] value The value obtained from given record's field.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_get_int(contacts_record_h rec_h, int id, int *value)
{
	int ret = contacts_record_get_int(rec_h, id, value);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_int() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function gets a string value from the given record field.
 * @param[in] rec_h The handle of the record whose string field value is to be obtained.
 * @param[in] id The identifier of the record's field.
 * @param[out] value The value obtained from given record's field.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_get_str(contacts_record_h rec_h, int id, char **value)
{
	char *temp = NULL;

	int ret = contacts_record_get_str_p(rec_h, id, &temp);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_int() failed. Err= %d.", ret);
	else if (temp)
		*value = strdup(temp);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function sets a boolean value to the given record field.
 * @param[in] rec_h The handle of the record whose boolean field value is to be set.
 * @param[in] id The identifier of the record's field.
 * @param[in] value The value to be set to the given record's field.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_set_bool(contacts_record_h rec_h, int id, bool value)
{
	int ret = contacts_record_set_bool(rec_h, id, value);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_set_bool() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function sets a integer value to the given record field.
 * @param[in] rec_h The handle of the record whose integer field value is to be set.
 * @param[in] id The identifier of the record's field.
 * @param[in] value The value to be set to the given record's field.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_set_int(contacts_record_h rec_h, int id, int value)
{
	int ret = contacts_record_set_int(rec_h, id, value);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_set_int() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function sets a string value to the given record field.
 * @param[in] rec_h The handle of the record whose string field value is to be set.
 * @param[in] id The identifier of the record's field.
 * @param[in] value The value to be set to the given record's field.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_set_str(contacts_record_h rec_h, int id, const char *value)
{
	int ret = contacts_record_set_str(rec_h, id, value);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_set_str() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function creates records list.
 * @param[out] rec_h The records list handle created.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_create_list(contacts_list_h *list_h)
{
	int ret = contacts_list_create(list_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_list_create() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function gets the number of records stored in a list.
 * @param[in] list_h The handle of the records list.
 * @param[out] count The number of records stored in a list.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_get_list_count(contacts_list_h list_h, int *count)
{
	int ret = contacts_list_get_count(list_h, count);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_list_get_count() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function moves the contacts list pointer to the first record.
 * @param[in] list_h The handle of the records list.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_set_list_first(contacts_list_h list_h)
{
	int ret = contacts_list_first(list_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_list_first() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Function add provided record to the list.
 * @param[in] list_h The handle of the list where the record is to be added.
 * @param[in] rec_h The handle of the record to be added to the list.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_common_add_list_record(contacts_list_h list_h, contacts_record_h rec_h)
{
	int ret = contacts_list_add(list_h, rec_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_list_add() failed. Err= %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Gets name of contact's field name by its type.
 * @param[in] field_type The type of the field.
 * @return The name of the field or NULL if the function fails.
 */
const char *data_common_get_field_name(sc_common_contact_fields field_type)
{
	if (field_type < CF_FIRST_NAME || field_type >= CF_MAX) {
		dlog_print(DLOG_ERROR, LOG_TAG, "utils_get_field_name() failed. Invalid input argument.");
		return NULL;
	}

	return _fields[field_type];
}

/**
 * @brief Retrieves the properties id for contact table.
 * @param[in] field_type Type of field: number, email
 * @param[out] parent_property_id Contact's property as key to child table
 * @param[out] child_uri URI of child table
 * @param[out] child_property_id ID of desired field of child table
 */
void data_common_get_contact_properties(sc_common_contact_fields field_type, unsigned *parent_property_id, const char **child_uri, unsigned *child_property_id)
{
	unsigned p_id = 0;
	unsigned c_id = 0;
	const char *uri = NULL;

	switch (field_type) {
	case CF_FIRST_NAME:
		p_id = _contacts_contact.name;
		uri = _contacts_name._uri;
		c_id = _contacts_name.first;
		break;
	case CF_LAST_NAME:
		p_id = _contacts_contact.name;
		uri = _contacts_name._uri;
		c_id = _contacts_name.last;
		break;
	case CF_PHONE_NUMBER:
		p_id = _contacts_contact.number;
		uri = _contacts_number._uri;
		c_id = _contacts_number.number;
		break;
	case CF_EMAIL:
		p_id = _contacts_contact.email;
		uri = _contacts_email._uri;
		c_id = _contacts_email.email;
		break;
	case CF_URL:
		p_id = _contacts_contact.url;
		uri = _contacts_url._uri;
		c_id = _contacts_url.url;
		break;
	case CF_COMPANY:
		p_id = _contacts_contact.company;
		uri = _contacts_company._uri;
		c_id = _contacts_company.name;
		break;
	case CF_BIRTHDAY:
		p_id = _contacts_contact.event;
		uri = _contacts_event._uri;
		c_id = _contacts_event.date;
		break;
	case CF_NOTE:
		p_id = _contacts_contact.note;
		uri = _contacts_note._uri;
		c_id = _contacts_note.note;
		break;
	case CF_MAX:
		break;
	}

	if (parent_property_id)
		*parent_property_id = p_id;

	if (child_property_id)
		*child_property_id = c_id;

	if (child_uri)
		*child_uri = uri;
}

/**
 * @brief Converts number representation of date to tm structure.
 * @param[in] date_value Date in contacts database format.
 * @return Date tm structure.
 */
struct tm data_common_int_to_date(int date_value)
{
	struct tm date = {
		.tm_mday = date_value % MONTH_FACTOR,
		.tm_mon = (date_value % YEAR_FACTOR) / MONTH_FACTOR - 1,
		.tm_year = date_value / YEAR_FACTOR - 1900
	};

	return date;
}

/**
 * @brief Convert date to string format.
 * @param[in] date The date tm structure or NULL for current date.
 * @return The date in string representation.
 */
char *data_common_date_to_str(struct tm *date)
{
	char *ret_str = malloc(DATE_STRING_LENGTH);
	if (ret_str)
		strftime(ret_str, DATE_STRING_LENGTH, "%x", date);

	return ret_str;
}

/**
 * @brief Trims spaces in provided string.
 * @param[in] str String to trim.
 * @return Trimmed string on success or NULL if the function fails.
 */
char *data_common_trim(const char *str)
{
	char *ret_str;
	Eina_Strbuf *strbuff = eina_strbuf_new();
	if (!strbuff) {
		dlog_print(DLOG_ERROR, LOG_TAG, "eina_strbuf_new() failed.");
		return NULL;
	}

	eina_strbuf_append(strbuff, str);
	eina_strbuf_trim(strbuff);
	ret_str = strdup(eina_strbuf_string_get(strbuff));
	eina_strbuf_free(strbuff);

	return ret_str;
}

/**
 *@brief Retrieves record list from contacts DB.
 *@remark return value MUST be destroyed with contacts_list_destroy by you.
 *@param[in] table_uri URI to table.
 *@param[in] filter_field Filterable field of table.
 *@param[in] filter_val Value of filterable field @a filter_field.
 *@return records list on success, otherwise NULL.
 */
contacts_list_h data_common_get_int_records(const char *table_uri, unsigned filter_field, int filter_val)
{
	contacts_list_h list = NULL;
	contacts_filter_h filter;

	if (!table_uri) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_common_get_int_records() failed. Invalid input argument.");
		return NULL;
	}

	filter = _create_int_filter(table_uri, filter_field, filter_val);

	if (filter) {
		list = _run_query(table_uri, filter);
		contacts_filter_destroy(filter);
	}

	return list;
}

/**
 *@brief Retrieves record list from contacts DB.
 *@remark return value MUST be destroyed with contacts_list_destroy by you.
 *@param[in] table_uri URI to table.
 *@param[in] filter_field Filterable field of table.
 *@param[in] filter_val Value of filterable field @a filter_field.
 *@return records list on success, otherwise NULL.
 */
contacts_list_h data_common_get_str_records(const char *table_uri, unsigned filter_field, const char *filter_val)
{
	contacts_list_h list = NULL;
	contacts_filter_h filter;

	if (!table_uri) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_common_get_str_records() failed. Invalid input argument.");
		return NULL;
	}

	filter = _create_str_filter(table_uri, filter_field, filter_val);

	if (filter) {
		list = _run_query(table_uri, filter);
		contacts_filter_destroy(filter);
	}

	return list;
}

/**
 *@brief Retrieves record list from contacts DB.
 *@remark return value MUST be destroyed with contacts_list_destroy by you.
 *@param[in] table_uri URI to table.
 *@param[in] filter_field Filterable field of table.
 *@param[in] filter_val Value of filterable field @a filter_field.
 *@return records list on success, otherwise NULL.
 */
contacts_list_h data_common_get_bool_records(const char *table_uri, unsigned filter_field, bool filter_val)
{
	contacts_list_h list = NULL;
	contacts_filter_h filter;

	if (!table_uri) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_common_get_bool_records() failed. Invalid input argument.");
		return NULL;
	}

	filter = _create_bool_filter(table_uri, filter_field, filter_val);

	if (filter) {
		list = _run_query(table_uri, filter);
		contacts_filter_destroy(filter);
	}

	return list;
}

/**
 * @brief Execute query.
 * @remark return value MUST be destroyed with contacts_list_destroy by you.
 * @param[in] table_uri URI to DB table.
 * @param[in] filter Filter for query.
 * @return list on success, otherwise NULL.
 */
static contacts_list_h _run_query(const char *table_uri, contacts_filter_h filter)
{
	int ret;
	contacts_list_h list = NULL;
	contacts_query_h query = NULL;

	if (!table_uri || !filter) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_common_run_query() failed. Invalid input argument.");
		return NULL;
	}

	ret = contacts_query_create(table_uri, &query);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_query_create() failed. Err = %d.", ret);
		return NULL;
	}

	ret = contacts_query_set_filter(query, filter);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_query_set_filter() failed. Err = %d.", ret);
		return NULL;
	}

	ret = contacts_db_get_records_with_query(query, 0, 0, &list);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_records_with_query() failed. Err = %d.", ret);

	contacts_query_destroy(query);

	return list;
}

/**
 * @brief Internal function which creates a filter.
 * @param[in] uri URI to DB table.
 * @param[out] filter Filter for query.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
static bool _create_filter(const char *uri, contacts_filter_h *filter)
{
	int ret;
	if (!uri) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_create_filter() failed. Invalid input argument.");
		return false;
	}

	ret = contacts_filter_create(uri, filter);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_filter_create() failed. Err = %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Internal function which creates a filter for integer fields.
 * @param[in] uri URI to DB table.
 * @param[in] filter_field The query field.
 * @param[in] int_val The integer filter value.
 * @return This function returns filter handle if the function succeeds,
 * otherwise NULL is returned.
 */
static contacts_filter_h _create_int_filter(const char *table_uri, unsigned int filter_field, int int_val)
{
	int ret;
	contacts_filter_h filter = NULL;

	if (!table_uri) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_create_int_filter() failed. Invalid input argument.");
		return NULL;
	}

	if (!_create_filter(table_uri, &filter))
		return NULL;

	ret = contacts_filter_add_int(filter, filter_field, CONTACTS_MATCH_EQUAL, int_val);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_filter_add_int() failed. Err = %d.", ret);
		contacts_filter_destroy(filter);
		filter = NULL;
	}

	return filter;
}

/**
 * @brief Internal function which creates a filter for string fields.
 * @param[in] uri URI to DB table.
 * @param[in] filter_field The query field.
 * @param[in] str_val The string filter value.
 * @return This function returns filter handle if the function succeeds,
 * otherwise NULL is returned.
 */
static contacts_filter_h _create_str_filter(const char *table_uri, unsigned int filter_field, const char *str_val)
{
	int ret;
	contacts_filter_h filter = NULL;

	if (!table_uri || !str_val) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_create_str_filter() failed. Invalid input argument.");
		return NULL;
	}

	if (!_create_filter(table_uri, &filter))
		return NULL;

	ret = contacts_filter_add_str(filter, filter_field, CONTACTS_MATCH_FULLSTRING, str_val);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_filter_add_str() failed. Err = %d.", ret);
		contacts_filter_destroy(filter);
		filter = NULL;
	}

	return filter;
}

/**
 * @brief Internal function which creates a filter for boolean fields.
 * @param[in] uri URI to DB table.
 * @param[in] filter_field The query field.
 * @param[in] bool_val The boolean filter value.
 * @return This function returns filter handle if the function succeeds,
 * otherwise NULL is returned.
 */
static contacts_filter_h _create_bool_filter(const char *table_uri, unsigned int filter_field, bool bool_val)
{
	int ret;
	contacts_filter_h filter = NULL;

	if (!table_uri) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_create_bool_filter() failed. Invalid input argument.");
		return NULL;
	}

	if (!_create_filter(table_uri, &filter))
		return NULL;

	ret = contacts_filter_add_bool(filter, filter_field, bool_val);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_filter_add_bool() failed. Err = %d.", ret);
		contacts_filter_destroy(filter);
		filter = NULL;
	}

	return filter;
}
