/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <contacts.h>
#include "contacts_app.h"
#include "data/data_group_edit.h"
#include "data/data_common.h"

/**
 * @brief Function retrieves a list which contains all the contacts which are not
 * present in a group described by provided identifier.
 * @param[in] group_id Id of group whose contacts needs to be excluded from the resulting list.
 * @return This function returns list of records if the function succeeds,
 * otherwise NULL is returned.
 */
contacts_list_h data_group_edit_get_contacts_to_add(int group_id)
{
	contacts_list_h contact_list = NULL;
	contacts_list_h members;
	contacts_record_h record;
	contacts_record_h contact_record;
	int contact_id = 0;
	int id = 0;

	if (!data_common_get_all_records(_contacts_contact._uri, &contact_list))
		return NULL;

	members = data_common_get_int_records(_contacts_group_relation._uri, _contacts_group_relation.group_id, group_id);

	data_common_set_list_first(members);
	CONTACTS_LIST_FOREACH(members, record) {
		if (!data_common_get_int(record, _contacts_group_relation.contact_id, &contact_id) || contact_id <= 0)
			continue;

		data_common_set_list_first(contact_list);
		CONTACTS_LIST_FOREACH(contact_list, contact_record) {
			if (!data_common_get_int(contact_record, _contacts_contact.id, &id) || contact_id != id)
				continue;

			contacts_list_remove(contact_list, contact_record);
			contacts_record_destroy(contact_record, true);
			break;
		}
	}

	contacts_list_destroy(members, true);
	contacts_list_first(contact_list);

	return contact_list;
}

/**
 * @brief Function creates a list of records to be deleted. The list consists
 * of all the records assigned to the group described by provided identifier.
 * @param[in] group_id Id of group whose contacts needs to be deleted.
 * @return This function returns list of records if the function succeeds,
 * otherwise NULL is returned.
 */
contacts_list_h data_group_edit_get_contacts_to_remove(int group_id)
{
	contacts_record_h record = NULL;
	contacts_list_h contact_list = NULL;
	contacts_list_h members;
	int contact_id = 0;

	if (!data_common_create_list(&contact_list))
		return NULL;

	members = data_common_get_int_records(_contacts_group_relation._uri, _contacts_group_relation.group_id, group_id);

	CONTACTS_LIST_FOREACH(members, record) {
		if (!data_common_get_int(record, _contacts_group_relation.contact_id, &contact_id) || contact_id <= 0)
			continue;

		if (!data_common_get_record(_contacts_contact._uri, contact_id, &record))
			continue;

		data_common_add_list_record(contact_list, record);
	}

	contacts_list_destroy(members, true);

	return contact_list;
}

/**
 * @brief Retrieves a list of group_edit_member_check_data.
 * @param[in] group_id Id of group.
 * @param[in] list Contacts list.
 * @return This function returns list of records if the function succeeds,
 * otherwise NULL is returned.
 */
Eina_List *data_group_edit_create_list_data(int group_id, contacts_list_h list)
{
	group_edit_member_check_data *check_data;
	Eina_List *result_list = NULL;
	contacts_record_h record = NULL;
	int contact_id = 0;
	char *name = NULL;

	if (!list) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_group_edit_create_list_data() failed. Invalid input argument.");
		return NULL;
	}

	CONTACTS_LIST_FOREACH(list, record) {
		if (!data_common_get_int(record, _contacts_contact.id, &contact_id) || contact_id <= 0)
			continue;

		check_data = (group_edit_member_check_data *)calloc(1, sizeof(group_edit_member_check_data));
		if (!check_data) {
			dlog_print(DLOG_ERROR, LOG_TAG, "calloc() failed.");
			continue;
		}

		check_data->contact_id = contact_id;
		check_data->is_checked = false;

		if (!data_common_get_str(record, _contacts_contact.display_name, &name))
			continue;

		if (name)
			check_data->name = strdup(name);

		result_list = eina_list_append(result_list, check_data);
	}

	return result_list;
}
