/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <efl_extension.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_contacts_list.h"

static struct view_info {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *navi;
} s_info = {
	.win = NULL,
	.conform = NULL,
	.navi = NULL,
};

static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info);
static void _navi_back_cb(void *data, Evas_Object *obj, void *event_info);
static void _context_back_cb(void *data, Evas_Object *obj, void *event_info);
static void _popup_cancel_cb(void *data, Evas_Object *obj, void *event_info);

/**
 * @brief Creates essential objects: window, conformant and layout.
 */
Eina_Bool view_create(void *user_data)
{
	/* Create the window */
	s_info.win = view_create_win(PACKAGE);
	if (s_info.win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a window.");
		return EINA_FALSE;
	}

	/* Create the conformant */
	s_info.conform = view_create_conformant(s_info.win);
	if (s_info.conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return EINA_FALSE;
	}

	s_info.navi = view_create_naviframe(s_info.conform, "elm.swallow.content");
	if (!s_info.navi)
		return false;

	if (!view_contacts_list_create(s_info.navi))
		return false;

	/* Show the window after main view is set up */
	evas_object_show(s_info.win);

	return EINA_TRUE;
}

/**
 * @brief Creates a basic window named package.
 * @param[in] pkg_name Name of the window
 */
Evas_Object *view_create_win(const char *pkg_name)
{
	Evas_Object *win = NULL;

	/*
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate the window.
	 */
	win = elm_win_util_standard_add(pkg_name, pkg_name);
	elm_win_conformant_set(win, EINA_TRUE);
	elm_win_indicator_mode_set(win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(win, ELM_WIN_INDICATOR_OPAQUE);
	elm_win_autodel_set(win, EINA_TRUE);

	evas_object_smart_callback_add(win, "delete,request", _delete_win_request_cb, NULL);

	return win;
}

/**
 * @brief Creates a layout object.
 * @param[in] parent The parent object for the layout.
 * @return The function returns layout object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_layout(Evas_Object *parent)
{
	Evas_Object *layout = elm_layout_add(parent);
	if (!layout) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_layout_add() failed.");
		return NULL;
	}

	elm_layout_theme_set(layout, "layout", "application", "default");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, 0);
	evas_object_show(layout);

	return layout;
}

/**
 * @brief Creates a conformant object.
 * @param[in] win The object to which you want to set this conformant
 * Conformant is mandatory for base GUI to have proper size
 */
Evas_Object *view_create_conformant(Evas_Object *win)
{
	/*
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);

	evas_object_show(conform);

	return conform;
}

/**
 * @brief Creates a naviframe for parent object and sets it to designated part.
 * @param[in] parent The parent object for naviframe.
 * @param[in] part_name The name of the part within parent object
 * where the naviframe is to be set,
 * @return This function returns a naviframe object if is was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_naviframe(Evas_Object *parent, const char *part_name)
{
	Evas_Object *navi = elm_naviframe_add(parent);
	if (!navi) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_naviframe_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(navi, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_naviframe_prev_btn_auto_pushed_set(navi, EINA_FALSE);
	elm_object_part_content_set(s_info.conform, part_name, navi);
	eext_object_event_callback_add(navi, EEXT_CALLBACK_BACK, _navi_back_cb, NULL);
	eext_object_event_callback_add(navi, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);

	evas_object_show(s_info.navi);

	return navi;
}

/**
 * @brief Creates a list for parent object.
 * @param[in] parent The parent object for a list.
 * @return This function returns a list object if is was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_list(Evas_Object *parent)
{
	Evas_Object *list = elm_list_add(parent);
	if (!list) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_list_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(list, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(list, EVAS_HINT_FILL, EVAS_HINT_FILL);

	elm_list_go(list);
	evas_object_show(list);

	return list;
}

/**
 * @brief Internal function which creates a genlist for parent object.
 * @param[in] parent The parent object for a genlist.
 * @return This function returns a genlist object if is was successfully created,
 * otherwise NULL is returned.
 */
Evas_Object *view_create_genlist(Evas_Object *parent)
{
	Evas_Object *genlist = elm_genlist_add(parent);
	if (!genlist) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);
	evas_object_show(genlist);

	return genlist;
}

/**
 * @brief Creates a button object for the parent object.
 * @param[in] parent The parent object for button object.
 * @param[in] caption The caption to be displayed on the button.
 * @param[in] part_name The part name where the button is to be set.
 * @param[in] on_click_cb The callback function's handler to be invoked on button click.
 * @param[in] data The data to be passed to the callback function.
 * @return The function returns button object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_button(Evas_Object *parent, const char *caption, const char *part_name, const char *style, Evas_Smart_Cb on_click_cb, void *data)
{
	Evas_Object *button = elm_button_add(parent);
	if (!button) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_button_add() failed.");
		return NULL;
	}

	evas_object_size_hint_weight_set(button, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(button, EVAS_HINT_FILL, EVAS_HINT_FILL);

	if (style)
		elm_object_style_set(button, style);

	if (caption)
		elm_object_text_set(button, caption);

	if (part_name)
		elm_object_part_content_set(parent, part_name, button);

	evas_object_smart_callback_add(button, "clicked", on_click_cb, data);
	evas_object_show(button);

	return button;
}

/**
 * @brief Creates a toolbar object for the parent object.
 * @param[in] parent The parent object for toolbar object.
 * @param[in] part_name The part name where the button is to be set.
 * @return The function returns toolbar object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_toolbar(Evas_Object *parent, const char *part_name)
{
	Evas_Object *toolbar = elm_toolbar_add(parent);
	if (!toolbar) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_toolbar_add() failed.");
		return NULL;
	}

	elm_toolbar_transverse_expanded_set(toolbar, EINA_TRUE);
	elm_toolbar_homogeneous_set(toolbar, EINA_TRUE);
	elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_select_mode_set(toolbar, ELM_OBJECT_SELECT_MODE_NONE);

	elm_layout_content_set(parent, part_name, toolbar);

	return toolbar;
}

/**
 * @brief Creates a context menu object for the parent object.
 * @param[in] parent The parent object for context menu object.
 * @return The function returns context menu object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_context_menu(Evas_Object *parent)
{
	Evas_Object *menu = elm_ctxpopup_add(parent);
	if (!menu) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_ctxpopup_add() failed.");
		return NULL;
	}

	elm_object_style_set(menu, "more/default");
	eext_object_event_callback_add(menu, EEXT_CALLBACK_BACK, _context_back_cb, NULL);
	eext_object_event_callback_add(menu, EEXT_CALLBACK_MORE, _context_back_cb, NULL);

	return menu;
}

/**
 * @brief Creates a popup window.
 * @param[in] parent The parent object for popup window.
 * @param[in] title The popup window's title.
 * @param[in] message The text message to be displayed inside the popup window.
 * @param[in] on_ok_cb The callback function to be invoked on 'OK' button press.
 * @return The function returns popup window object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_popup(Evas_Object *parent, const char *title, const char *message, Evas_Smart_Cb on_ok_cb)
{
	Evas_Object *popup = elm_popup_add(parent);
	if (!popup) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_popup_add() failed.");
		return NULL;
	}

	if (message)
		elm_object_text_set(popup, message);

	elm_object_part_text_set(popup, "title,text", title);
	eext_object_event_callback_add(popup, EEXT_CALLBACK_BACK, eext_popup_back_cb, NULL);

	if (on_ok_cb) {
		view_create_button(popup, "OK", "button1", NULL, on_ok_cb, (void *)popup);
		view_create_button(popup, "Cancel", "button2", NULL, _popup_cancel_cb, (void *)popup);
	} else {
		view_create_button(popup, "Close", "button1", NULL, _popup_cancel_cb, (void *)popup);
	}

	evas_object_show(popup);

	return popup;
}

/**
 * @brief Creates an edit field embedded into a layout.
 * @param[in] parent The parent object for popup window.
 * @param[in] guide_text The text displayed when the entry widget is empty.
 * @param[in] multiline The single or multi line style.
 * @return The function returns layout object containing the entry widget
 * if it was created successfully, otherwise 'NULL' is returned.
 */
Evas_Object *view_create_edit_field(Evas_Object *parent, const char *guide_text, bool multiline)
{
	Evas_Object *entry;
	Evas_Object *layout = view_create_layout(parent);
	if (!layout)
		return NULL;

	entry = elm_entry_add(layout);
	if (!entry) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_entry_add() failed.");
		return NULL;
	}

	elm_entry_single_line_set(entry, !(Eina_Bool)multiline);
	elm_entry_scrollable_set(entry, EINA_FALSE);

	if (multiline)
		elm_layout_theme_set(layout, "layout", "editfield", "multiline");
	else
		elm_layout_theme_set(layout, "layout", "editfield", "singleline");

	elm_object_part_text_set(entry, "elm.guide", guide_text);
	elm_object_part_content_set(layout, "elm.swallow.content", entry);
	evas_object_show(layout);

	return layout;
}

/**
 * @brief Creates a date/time widget.
 * @param[in] parent The parent object for date/time widget.
 * @param[in] date The initial date to be displayed.
 * @return The function returns date/time object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_datetime(Evas_Object *parent, struct tm date)
{
	Evas_Object *datetime = elm_datetime_add(parent);
	if (!datetime) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_datetime_add() failed.");
		return NULL;
	}

	elm_datetime_format_set(datetime, "%%x");
	elm_datetime_value_set(datetime, &date);
	elm_object_content_set(parent, datetime);
	evas_object_show(datetime);

	return datetime;
}

/**
 * @brief Creates a check widget.
 * @param[in] parent The parent object for check widget.
 * @return The function returns check object if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Evas_Object *view_create_check(Evas_Object *parent, Eina_Bool *is_checked)
{
	Evas_Object *check = elm_check_add(parent);
	if (!check) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_check_add() failed.");
		return NULL;
	}

	elm_check_state_set(check, *is_checked);
	evas_object_propagate_events_set(check, EINA_FALSE);
	elm_check_state_pointer_set(check, is_checked);

	return check;
}

/**
 * @brief Function which creates an item class for genlist object.
 * @param[in] item_style The name of the style to be applied to the item.
 * @param[in] on_text_get_cb The callback function invoked on item's redraw.
 * This callback function is assigned to the func.text_get member of the genlist item class structure.
 * @param[in] on_content_get_cb The callback function invoked on item's redraw.
 * This callback function is assigned to the func.content_get member of the genlist item class structure.
  * @return The function returns item class if it was created successfully,
 * otherwise 'NULL' is returned.
 */
Elm_Genlist_Item_Class *create_genlist_item_class(const char *item_style, Elm_Genlist_Item_Text_Get_Cb on_text_get_cb, Elm_Genlist_Item_Content_Get_Cb on_content_get_cb)
{
	Elm_Genlist_Item_Class *itc = elm_genlist_item_class_new();
	if (!itc) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_genlist_item_class_new() failed.");
		return NULL;
	}

	itc->item_style = item_style;
	itc->func.text_get = on_text_get_cb;
	itc->func.content_get = on_content_get_cb;
	itc->func.state_get = NULL;
	itc->func.del = NULL;

	return itc;
}

/**
 * @brief Displays prior created context menu aligned to the given object.
 * @param[in] menu The context menu object.
 * @param[in] align_to The widget to which the context menu needs to be aligned.
 */
void view_display_context_menu(Evas_Object *menu, Evas_Object *align_to)
{
	int y;
	int height;

	evas_object_geometry_get(align_to, NULL, &y, NULL, &height);
	evas_object_move(menu, 0, y + height);
	evas_object_show(menu);
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_destroy(void)
{
	if (s_info.win == NULL)
		return;

	evas_object_del(s_info.win);
}

/**
 * @brief Internal callback function invoked when the main window needs to be destroyed.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_win_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _navi_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (elm_naviframe_top_item_get(obj) == elm_naviframe_bottom_item_get(obj))
		/* Collapse application window. To exit use ui_app_exit() instead */
		elm_win_lower(s_info.win);
	else
		elm_naviframe_item_pop(obj);
}

/**
 * @brief Internal callback function invoked on HW Back button press.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _context_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_ctxpopup_dismiss(obj);
}

/**
 * @brief Internal callback function invoked on 'Cancel' button press from the popup window.
 * @param[in] data The user data passed to the eext_object_event_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del((Evas_Object *)data);
}
