/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <time.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_contact_edit.h"
#include "view/view_common.h"
#include "data/data_contact_edit.h"
#include "data/data_common.h"

#define YEAR_BASE 1900
#define YEAR_FACTOR 10000
#define MONTH_BASE 1
#define MONTH_FACTOR 100

static struct view_info {
	Evas_Object *navi;
	int contact_id;
	sc_ec_contact_edit *data;
	Evas_Object *genlist;
	Elm_Object_Item *birthday_item;
	Evas_Object *datetime;
} s_info = {
	.navi = NULL,
	.contact_id = 0,
	.data = NULL,
	.genlist = NULL,
	.birthday_item = NULL,
	.datetime = NULL,
};

static void _view_changed_cb(const char *view_uri, void *user_data);
static Evas_Object *_entry_content_get_cb(void *data, Evas_Object *obj, const char *part);
static Evas_Object *_date_content_get_cb(void *data, Evas_Object *obj, const char *part);
static void _list_destroy_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _entry_unfocused_cb(void *data, Evas_Object *obj, void *event_info);
static void _date_changed_cb(void *data, Evas_Object *obj, void *event_info);
static void _date_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _save_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _populate_list(void);
static void _set_keyboard_layout(Evas_Object *entry, sc_common_contact_fields field_type);
static int _date_to_int(const struct tm *date);

/**
 * @brief Creates contact's edit view.
 * @param[in] parent Parent object.
 * @param[in] contact_id ID of contact.
 * @return This function returns 'EINA_TRUE' if the view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_contact_edit_create(Evas_Object *parent, int contact_id)
{
	int ret;
	char *caption;
	Evas_Object *toolbar;

	if (!parent) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid input argument for contact edit view.");
		return EINA_FALSE;
	}

	if (contact_id < 0)
		contact_id = 0;

	s_info.navi = parent;
	s_info.contact_id = contact_id;

	if (contact_id > 0)
		s_info.data = data_contact_edit_get(s_info.contact_id);
	else
		s_info.data = data_contact_edit_new();

	if (!s_info.data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid contact data.");
		_list_destroy_cb(NULL, NULL, NULL, NULL);
		return EINA_FALSE;
	}

	s_info.genlist = view_create_genlist(s_info.navi);
	if (!s_info.genlist)
		return EINA_FALSE;

	evas_object_event_callback_add(s_info.genlist, EVAS_CALLBACK_FREE, _list_destroy_cb, NULL);
	_populate_list();

	caption = contact_id > 0 ? "Edit contact" : "Create contact";
	elm_naviframe_item_push(s_info.navi, caption, NULL, NULL, s_info.genlist, "tabbar");

	toolbar = view_create_toolbar(s_info.navi, "toolbar");
	if (!toolbar)
		return EINA_FALSE;

	elm_toolbar_item_append(toolbar, NULL, "Save", _save_button_click_cb, NULL);

	ret = contacts_db_add_changed_cb(_contacts_person._uri, _view_changed_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed. Err= %d.", ret);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on contact update.
 * This function returns to contacts list view.
 * @param[in] view_uri The URI of the view.
 * @param[in] user_data The data passed to the callback attachment function.
 */
static void _view_changed_cb(const char *view_uri, void *user_data)
{
	view_common_return_to_view(s_info.navi, VI_PERSON_LIST);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains an edit field built over the layout.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_entry_content_get_cb(void *data, Evas_Object *obj, const char *part)
{
	char *text = NULL;
	const char *guide_text;
	Evas_Object *edit_field;
	Evas_Object *entry;
	sc_common_contact_fields field_type = (sc_common_contact_fields)data;
	if (field_type >= CF_MAX || field_type < CF_FIRST_NAME) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_entry_content_get_cb() failed. Incorrect field type.");
		return NULL;
	}

	if (!s_info.data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_entry_content_get_cb() failed. Invalid contact record.");
		return NULL;
	}

	guide_text = data_common_get_field_name(field_type);
	edit_field = view_create_edit_field(s_info.genlist, guide_text, field_type == CF_NOTE);
	if (!edit_field)
		return NULL;

	entry = elm_object_part_content_get(edit_field, "elm.swallow.content");
	elm_entry_scrollable_set(entry, EINA_TRUE);

	_set_keyboard_layout(entry, field_type);

	if (data_contact_edit_get_str_value(field_type, s_info.data->contact_record, &text))
		elm_entry_entry_set(entry, text);

	evas_object_smart_callback_add(entry, "unfocused", _entry_unfocused_cb, (void *)field_type);

	return edit_field;
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * The item's area contains a button widget.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static Evas_Object *_date_content_get_cb(void *data, Evas_Object *obj, const char *part)
{
	struct tm date = data_contact_edit_get_date(s_info.data->contact_record);
	char *str_date = data_common_date_to_str(&date);
	Evas_Object *button = view_create_button(obj, str_date, NULL, NULL, _date_button_click_cb, NULL);

	free(str_date);

	return button;
}

/**
 * @brief Internal callback function invoked on genlist deletion.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] e The Evas surface of the object.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 * @return The text string to be displayed is returned.
 */
static void _list_destroy_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	data_contact_edit_destroy(s_info.data);
	contacts_db_remove_changed_cb(_contacts_contact._uri, _view_changed_cb, NULL);
}

/**
 * @brief Internal callback function which is responsible for provided text trimming
 * and storing. This callback function is invoked when the entry field becomes unfocused.
 * @param[in] data The user data passed to the evas_object_smart_callback_add() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _entry_unfocused_cb(void *data, Evas_Object *obj, void *event_info)
{
	char *text_trim;
	const char *text;
	sc_common_contact_fields field_type = (sc_common_contact_fields)data;
	if (field_type >= CF_MAX || field_type < CF_FIRST_NAME) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_entry_unfocused_cb() failed. Incorrect field type.");
		return;
	}

	if (!s_info.data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_entry_unfocused_cb() failed. Invalid contact record.");
		return;
	}

	text = elm_entry_entry_get(obj);
	text_trim = data_common_trim(text);

	data_contact_edit_set_str_value(field_type, s_info.data->contact_record, text_trim);

	free(text_trim);
}

/**
 * @brief Internal callback function which is responsible for changed date handling.
 * This callback function is invoked when the date field is updated.
 * @param[in] data The user data passed to the view_create_popup() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _date_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct tm date;
	Evas_Object *popup = (Evas_Object *)data;

	if (!s_info.data)
		dlog_print(DLOG_ERROR, LOG_TAG, "_date_changed_cb() failed. Invalid contact data.");
	else if (!elm_datetime_value_get(s_info.datetime, &date))
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_datetime_value_get() failed.");
	else
		data_contact_edit_set_int_value(CF_BIRTHDAY, s_info.data->contact_record, _date_to_int(&date));

	evas_object_del(popup);
	elm_genlist_item_update(s_info.birthday_item);
}

/**
 * @brief Internal callback function which is responsible for popup window creation
 * with date setting widget inside. This callback function is invoked on the
 * birthday date button press.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _date_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct tm date = data_contact_edit_get_date(s_info.data->contact_record);
	Evas_Object *popup = view_create_popup(s_info.navi, "Birthday", NULL, _date_changed_cb);
	if (!popup)
		return;

	s_info.datetime = view_create_datetime(popup, date);
	if (!s_info.datetime) {
		evas_object_del(popup);
		return;
	}

	evas_object_show(popup);
}

/**
 * @brief Internal callback function which is responsible for edited contact storage.
 * This callback function is invoked on "Save" button press.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _save_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (!s_info.data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "_save_button_click_cb() failed. Invalid contact data.");
		return;
	}

	if (!data_contact_edit_check_contact_name(s_info.data->contact_record)) {
		view_create_popup(s_info.navi, "Information", "Enter the first name or the last name", NULL);
		return;
	}

	if (!data_contact_edit_save(s_info.data))
		dlog_print(DLOG_ERROR, LOG_TAG, "sc_ec_save_contact() failed. Contact not saved.");

	elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal function which populates the view with selected contact's
 * detailed information.
 */
static void _populate_list(void)
{
	Elm_Genlist_Item_Class *itc_birthday = create_genlist_item_class("full", NULL, _date_content_get_cb);
	Elm_Genlist_Item_Class *itc_entry = create_genlist_item_class("full", NULL, _entry_content_get_cb);
	sc_common_contact_fields i = CF_FIRST_NAME;

	for (; i < CF_MAX; ++i)
		if (i == CF_BIRTHDAY)
			s_info.birthday_item = elm_genlist_item_append(s_info.genlist, itc_birthday, (void *)i, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		else
			elm_genlist_item_append(s_info.genlist, itc_entry, (void *)i, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);

	elm_genlist_item_class_free(itc_birthday);
	elm_genlist_item_class_free(itc_entry);
}

/**
 * @brief Internal function which sets the software input panel (keyboard) layout.
 * @param[in] entry The entry object associated with the input panel.
 * @param[in] field_type The type of information to be input to the entry widget.
 */
static void _set_keyboard_layout(Evas_Object *entry, sc_common_contact_fields field_type)
{
	switch (field_type) {
	case CF_PHONE_NUMBER:
		elm_entry_input_panel_layout_set(entry, ELM_INPUT_PANEL_LAYOUT_PHONENUMBER);
		break;
	case CF_EMAIL:
		elm_entry_input_panel_layout_set(entry, ELM_INPUT_PANEL_LAYOUT_EMAIL);
		break;
	case CF_URL:
		elm_entry_input_panel_layout_set(entry, ELM_INPUT_PANEL_LAYOUT_URL);
		break;
	default:
		break;
	}
}

/**
 * @brief Convert date to contacts database format.
 * @param[in] date The date tm structure or NULL for current date.
 * @return Number representation of date.
 */
static int _date_to_int(const struct tm *date)
{
	if (!date) {
		time_t current_time = time(NULL);
		date = localtime(&current_time);
	}

	return (date->tm_year + YEAR_BASE) * YEAR_FACTOR + (date->tm_mon + MONTH_BASE) * MONTH_FACTOR +  date->tm_mday;
}
