/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_favorites_list.h"
#include "view/view_contact_details.h"
#include "data/data_common.h"

static struct view_info {
	Evas_Object *navi;
	Evas_Object *list;
	contacts_list_h fav_list;
} s_info = {
	.navi = NULL,
	.list = NULL,
	.fav_list = NULL,
};

static void _destroy_list_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _item_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _update_list_cb(const char *view_uri, void *user_data);
static void _populate_list(void);

/**
 * @brief Add favorites list view.
 * @param[in] parent Parent object.
 * @return This function returns 'EINA_TRUE' if the view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_favourites_list_create(Evas_Object *parent)
{
	int ret;

	if (!parent) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid input argument for favorites view.");
		return EINA_FALSE;
	}

	s_info.navi = parent;
	s_info.list = view_create_list(s_info.navi);
	if (!s_info.list)
		return EINA_FALSE;

	evas_object_event_callback_add(s_info.list, EVAS_CALLBACK_FREE, _destroy_list_cb, NULL);
	elm_naviframe_item_push(s_info.navi, "Favorites", NULL, NULL, s_info.list, NULL);

	_populate_list();

	ret = contacts_db_add_changed_cb(_contacts_person._uri, _update_list_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed. Err= %d.", ret);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on favorites list destroy.
 * This function removes the 'changed' callback attached at the view creation
 * and deletes the contacts list.
 * @param[in] data The user data passed to the contacts_db_add_changed_cb() function.
 * @param[in] e Evas surface of the object.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _destroy_list_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	contacts_list_destroy(s_info.fav_list, true);
	contacts_db_remove_changed_cb(_contacts_person._uri, _update_list_cb, NULL);
}

/**
 * @brief Internal callback function invoked on favorites list item selection.
 * This function displays the selected contact details.
 * @param[in] data The user data passed to the elm_list_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _item_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	int ret;
	int person_id = 0;
	contacts_record_h record = (contacts_record_h)data;

	elm_list_item_selected_set(event_info, EINA_FALSE);

	ret = contacts_record_get_int(record, _contacts_person.id, &person_id);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_int() failed. Err = %d", ret);
		return;
	}

	view_contact_details_create(s_info.navi, person_id);
}

/**
 * @brief Internal callback function invoked on favorite contacts list update.
 * This function removes all items from the list and populates it again.
 * @param[in] view_uri The URI of the view.
 * @param[in] user_data The data passed to the callback attachment function.
 */
static void _update_list_cb(const char *view_uri, void *user_data)
{
	elm_list_clear(s_info.list);
	_populate_list();
}

/**
 * @brief Internal function which adds favorite contacts to the list.
 */
static void _populate_list(void)
{
	char *name = NULL;
	contacts_record_h record = NULL;

	s_info.fav_list = data_common_get_bool_records(_contacts_person._uri, _contacts_person.is_favorite, true);
	if (!s_info.fav_list)
		return;

	CONTACTS_LIST_FOREACH(s_info.fav_list, record)
		if (contacts_record_get_str_p(record, _contacts_person.display_name, &name) == CONTACTS_ERROR_NONE)
			elm_list_item_append(s_info.list, name, NULL, NULL, _item_selected_cb, record);
		else
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_str_p() failed.");

	elm_list_go(s_info.list);
}
