/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_group_create.h"
#include "data/data_group_create.h"
#include "data/data_common.h"

static struct view_info {
	Evas_Object *navi;
	Evas_Object *entry;
} s_info = {
	.navi = NULL,
	.entry = NULL,
};

static Evas_Object *_create_editfield(void);
static void _save_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _cancel_button_click_cb(void *data, Evas_Object *obj, void *event_info);

/**
 * @brief Creates category creation view.
 * @param[in] parent Parent object.
 * @return This function returns 'EINA_TRUE' if the view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_group_create_create(Evas_Object *parent)
{
	Evas_Object *toolbar;
	Evas_Object *editfield_layout;

	if (!parent) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid input argument for group creation view.");
		return EINA_FALSE;
	}

	s_info.navi = parent;

	editfield_layout = _create_editfield();
	if (!editfield_layout)
		return EINA_FALSE;

	elm_naviframe_item_push(s_info.navi, "Categories", NULL, NULL, editfield_layout, NULL);

	toolbar = view_create_toolbar(s_info.navi, "toolbar");
	if (!toolbar)
		return EINA_FALSE;

	elm_toolbar_item_append(toolbar, NULL, "Save", _save_button_click_cb, NULL);
	elm_toolbar_item_append(toolbar, NULL, "Cancel", _cancel_button_click_cb, NULL);

	return EINA_TRUE;
}

/**
 * @brief Internal function which creates edit field embedded into a layout object
 * with white background set.
 * @return This function returns top layout object if the entire edit field
 * composition was was created successfully, otherwise 'EINA_FALSE' is returned.
 */
static Evas_Object *_create_editfield(void)
{
	Evas_Object *bg;
	Evas_Object *box;
	Evas_Object *editfield;
	Evas_Object *layout = view_create_layout(s_info.navi);
	if (!layout)
		return NULL;

	bg = elm_bg_add(layout);
	if (!bg) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_bg_add() failed.");
		return NULL;
	}

	elm_bg_color_set(bg, 255, 255, 255);
	elm_object_part_content_set(layout, "elm.swallow.bg", bg);

	box = elm_box_add(layout);
	if (!box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "elm_box_add() failed.");
		return NULL;
	}

	elm_object_part_content_set(layout, "elm.swallow.content", box);

	editfield = view_create_edit_field(box, "Category name", false);
	if (!editfield)
		return NULL;

	elm_box_pack_end(box, editfield);

	s_info.entry = elm_object_part_content_get(editfield, "elm.swallow.content");

	return layout;
}

/**
 * @brief Internal callback function invoked on 'Save' button click.
 * This function is responsible for input name validation and group creation
 * if it does not exist. On failure, the popup window is displayed.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _save_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	bool group_exists = false;
	const char *name = data_common_trim(elm_object_text_get(s_info.entry));

	if (!data_group_create_exists(name, &group_exists))
		return;

	if (group_exists) {
		view_create_popup(s_info.navi, "Error", "Group already exists", NULL);
		return;
	}

	if (!data_group_create_create(name)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failed to create '%s' group.", name);
		return;
	}

	elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal callback function invoked on 'Cancel' button click.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _cancel_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_naviframe_item_pop(s_info.navi);
}
