/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <contacts.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_group_details.h"
#include "view/view_contact_details.h"
#include "view/view_group_edit_member.h"
#include "view/view_common.h"
#include "data/data_common.h"

static struct view_info {
	Evas_Object *navi;
	Elm_Object_Item *navi_item;
	Evas_Object *list;
	int group_id;
} s_info = {
	.navi = NULL,
	.navi_item = NULL,
	.list = NULL,
	.group_id = 0,
};

static void _add_member_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _del_member_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _delete_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _popup_ok_button_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _item_selected_cb(void *data, Evas_Object *obj, void *event_info);
static void _update_details_cb(const char *view_uri, void *user_data);
static void _update_relation_details_cb(const char *view_uri, void *user_data);
static bool _get_group_name(int group_id, char **group_name);
static void _populate_list(void);

/**
 * @brief Creates group details view.
 * @param[in] parent Parent object.
 * @param[in] group_id The identifier of a group.
 * @return This function returns 'EINA_TRUE' if the view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_group_details_create(Evas_Object *parent, int group_id)
{
	int ret;
	char *group_name = NULL;
	Evas_Object *toolbar;

	if (!parent || group_id <= 0) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid input argument for group details view.");
		return EINA_FALSE;
	}

	if (!_get_group_name(group_id, &group_name))
		return EINA_FALSE;

	s_info.navi = parent;
	s_info.group_id = group_id;
	s_info.list = view_create_list(s_info.navi);
	if (!s_info.list) {
		free(group_name);
		return EINA_FALSE;
	}

	_populate_list();

	s_info.navi_item = elm_naviframe_item_push(s_info.navi, group_name, NULL, NULL, s_info.list, NULL);
	free(group_name);

	toolbar = view_create_toolbar(s_info.navi, "toolbar");
	if (!toolbar)
		return EINA_FALSE;

	elm_toolbar_item_append(toolbar, NULL, "Add member", _add_member_button_click_cb, NULL);
	elm_toolbar_item_append(toolbar, NULL, "Remove member", _del_member_button_click_cb, NULL);

	view_create_button(s_info.navi, "Delete", "title_right_btn", "naviframe/title_right", _delete_button_click_cb, NULL);

	ret = contacts_db_add_changed_cb(_contacts_group._uri, _update_details_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed for group. Err = %d", ret);

	ret = contacts_db_add_changed_cb(_contacts_group_relation._uri, _update_relation_details_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed for group relation. Err = %d", ret);

	ret = contacts_db_add_changed_cb(_contacts_person._uri, _update_relation_details_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed for group person. Err = %d", ret);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on 'Add member' button click.
 * This function switches to the contact selection view.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _add_member_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	view_group_edit_member_create(s_info.navi, s_info.group_id, EDIT_MEMBER_TYPE_ADD);
}

/**
 * @brief Internal callback function invoked on 'Remove member' button click.
 * This function switches to the contact selection view.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _del_member_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	int count = 0;
	contacts_list_h list;

	list = data_common_get_int_records(_contacts_group_relation._uri, _contacts_group_relation.group_id, s_info.group_id);
	contacts_list_get_count(list, &count);
	contacts_list_destroy(list, true);

	if (count <= 0)
		return;

	view_group_edit_member_create(s_info.navi, s_info.group_id, EDIT_MEMBER_TYPE_REMOVE);
}

/**
 * @brief Internal callback function invoked on 'Delete' button click.
 * This function displays the prompt for group deletion.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	view_create_popup(s_info.navi, "Caution", "Do you want to remove this category?", _popup_ok_button_click_cb);
}

/**
 * @brief Internal callback function invoked on 'Ok' button click in the popup window
 * prompting for the selected category removal confirmation.
 * This function removes the category and returns to the categories view.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_ok_button_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	int ret;
	Evas_Object *popup = (Evas_Object *)data;

	evas_object_del(popup);

	ret = contacts_db_delete_record(_contacts_group._uri, s_info.group_id);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_delete_record() failed. Err = %d.", ret);

	elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal callback function invoked on list item selection.
 * This function switches to the contact details view.
 * @param[in] data The user data passed to the elm_list_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _item_selected_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_list_item_selected_set((Elm_Object_Item *)event_info, EINA_FALSE);
	view_contact_details_create(s_info.navi, (int)data);
}

/**
 * @brief Internal callback function invoked on group details update.
 * This function updates the displayed group name if it changes.
 * @param[in] view_uri URI of the database view.
 * @param[in] user_data User data passed to the contacts_db_add_changed_cb() function.
 */
static void _update_details_cb(const char *view_uri, void *user_data)
{
	int ret;
	char *name = NULL;
	contacts_record_h rec_h = NULL;

	ret = contacts_db_get_record(_contacts_group._uri, s_info.group_id, &rec_h);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_record() failed. Err = %d.", ret);

	if (!rec_h) {
		view_common_return_to_view(s_info.navi, VI_CATEGORIES_LIST);
		return;
	}

	ret = contacts_record_get_str_p(rec_h, _contacts_group.name, &name);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_str_p() failed. Err = %d.", ret);

	elm_object_item_text_set(s_info.navi_item, name);
	contacts_record_destroy(rec_h, true);
}

/**
 * @brief Internal callback function invoked on group's related item details update.
 * This function updates the displayed item details if it changes.
 * @param[in] view_uri URI of the database view.
 * @param[in] user_data User data passed to the contacts_db_add_changed_cb() function.
 */
static void _update_relation_details_cb(const char *view_uri, void *user_data)
{
	elm_list_clear(s_info.list);

	_populate_list();
}

/**
 * @brief Internal function which obtains the name of a group for given ID.
 * @param[in] group_id ID of a group.
 * @param[out] group_name The name for given group ID.
 * @return This function returns 'true' if the group name was obtained successfully,
 * otherwise 'false' is returned.
 */
static bool _get_group_name(int group_id, char **group_name)
{
	int ret;
	contacts_record_h rec_h;

	ret = contacts_db_get_record(_contacts_group._uri, group_id, &rec_h);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_record() failed. Err = %d.", ret);
		return false;
	}

	ret = contacts_record_get_str(rec_h, _contacts_group.name, group_name);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_str_p() failed. Err = %d.", ret);

	contacts_record_destroy(rec_h, false);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Internal function which populates the selected contact list with
 * relevant detailed information.
 */
static void _populate_list(void)
{
	contacts_record_h rec_h = NULL;
	contacts_list_h members;
	char *name = NULL;
	int contact_id = 0;
	int person_id = 0;
	int ret;

	members = data_common_get_int_records(_contacts_group_relation._uri, _contacts_group_relation.group_id, s_info.group_id);

	CONTACTS_LIST_FOREACH(members, rec_h) {
		ret = contacts_record_get_int(rec_h, _contacts_group_relation.contact_id, &contact_id);
		if (ret != CONTACTS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_int() failed. Err = %d.", ret);
			continue;
		}

		ret = contacts_db_get_record(_contacts_contact._uri, contact_id, &rec_h);
		if (ret != CONTACTS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_record() failed. Err = %d.", ret);
			continue;
		}

		ret = contacts_record_get_str_p(rec_h, _contacts_contact.display_name, &name);
		if (ret != CONTACTS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_str_p() failed. Err = %d.", ret);
			continue;
		}

		ret = contacts_record_get_int(rec_h, _contacts_contact.person_id, &person_id);
		if (ret != CONTACTS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_int() failed. Err = %d.", ret);
			continue;
		}

		elm_list_item_append(s_info.list, name, NULL, NULL, _item_selected_cb, (void *)person_id);

		ret = contacts_record_destroy(rec_h, true);
		if (ret != CONTACTS_ERROR_NONE)
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_destroy() failed. Err = %d.", ret);
	}

	ret = contacts_list_destroy(members, true);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_list_destroy() failed. Err = %d.", ret);

	elm_list_go(s_info.list);
}
