/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <contacts.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_group_edit_member.h"
#include "data/data_group_edit.h"

static struct view_info {
	Evas_Object *navi;
	Evas_Object *genlist;
	Eina_List *check_list;
	int group_id;
	edit_type_e type;
} s_info = {
	.navi = NULL,
	.genlist = NULL,
	.check_list = NULL,
	.group_id = 0,
	.type = EDIT_MEMBER_TYPE_ADD,
};

static char *_genlist_text_get_cb(void *data, Evas_Object *obj, const char *part);
static Evas_Object *_genlist_content_get_cb(void *data, Evas_Object *obj, const char *part);
static void _genlist_destroy_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _edit_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _populate_list(void);
static void _display_popup(void);

/**
 * @brief Creates the group edit member view.
 * @param[in] parent Parent object.
 * @param[in] group_record Record of the group for which the details view will be created.
 * @param[in] type Edit type.
 * @return This function returns 'EINA_TRUE' if the view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_group_edit_member_create(Evas_Object *parent, int group_id, edit_type_e type)
{
	Evas_Object *toolbar;
	const char *item_caption;

	if (!parent || group_id <= 0) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid input argument for group details view.");
		return EINA_FALSE;
	}

	s_info.navi = parent;
	s_info.group_id = group_id;
	s_info.type = type;
	s_info.genlist = view_create_genlist(s_info.navi);
	if (!s_info.genlist)
		return EINA_FALSE;

	evas_object_event_callback_add(s_info.genlist, EVAS_CALLBACK_FREE, _genlist_destroy_cb, NULL);

	_populate_list();

	item_caption = type == EDIT_MEMBER_TYPE_ADD ? "Add Member" : type == EDIT_MEMBER_TYPE_REMOVE ? "Remove Member" : "";

	elm_naviframe_item_push(s_info.navi, item_caption, NULL, NULL, s_info.genlist, NULL);

	toolbar = view_create_toolbar(s_info.navi, "toolbar");
	if (!toolbar)
		return EINA_FALSE;

	elm_toolbar_item_append(toolbar, NULL, item_caption, _edit_button_clicked_cb, NULL);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * The item's area consists of one label which displays the contact's name.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_genlist_text_get_cb(void *data, Evas_Object *obj, const char *part)
{
	group_edit_member_check_data *check_data = (group_edit_member_check_data *)data;

	if (!strcmp(part, "elm.text"))
		if (check_data->name)
			return elm_entry_utf8_to_markup(check_data->name);

	return NULL;
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.content_get handler of genlist item's class structure.
 * In the item's content area the check widget is displayed.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The check widget is returned.
 */
static Evas_Object *_genlist_content_get_cb(void *data, Evas_Object *obj, const char *part)
{
	Evas_Object *check = NULL;
	group_edit_member_check_data *check_data = (group_edit_member_check_data *)data;

	if (!strcmp(part, "elm.swallow.icon"))
		check = view_create_check(obj, &check_data->is_checked);

	return check;
}

/**
 * @brief Internal callback function invoked on genlist object removal.
 * @param[in] data The user data passed to the evas_object_event_callback_add() function.
 * @param[in] e The evas surface of the object.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _genlist_destroy_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	group_edit_member_check_data *check_data;

	EINA_LIST_FREE(s_info.check_list, check_data) {
		if (!check_data)
			continue;

		if (check_data->name)
			free(check_data->name);

		free(check_data);
	}
}

/**
 * @brief Internal callback function invoked on 'Add/Remove member' button click.
 * This function adds or removes the contact from the category.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _edit_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	Eina_List *l = NULL;
	void *ch_data = NULL;
	bool is_checked = false;
	group_edit_member_check_data *check_data;

	if (!s_info.check_list) {
		_display_popup();
		return;
	}

	EINA_LIST_FOREACH(s_info.check_list, l, ch_data) {
		if (!ch_data)
			continue;

		check_data = (group_edit_member_check_data *)ch_data;
		if (!check_data->is_checked)
			continue;

		is_checked = true;
		if (s_info.type == EDIT_MEMBER_TYPE_ADD)
			contacts_group_add_contact(s_info.group_id, check_data->contact_id);
		else if (s_info.type == EDIT_MEMBER_TYPE_REMOVE)
			contacts_group_remove_contact(s_info.group_id, check_data->contact_id);
	}

	if (!is_checked)
		_display_popup();
	else
		elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal function which populates the selected contact list with
 * relevant detailed information.
 */
static void _populate_list(void)
{
	Eina_List *l = NULL;
	void *check_data = NULL;
	contacts_list_h list = NULL;

	Elm_Genlist_Item_Class *itc = create_genlist_item_class("1line", _genlist_text_get_cb, _genlist_content_get_cb);
	if (!itc)
		return;

	if (s_info.type == EDIT_MEMBER_TYPE_ADD)
		list = data_group_edit_get_contacts_to_add(s_info.group_id);
	else if (s_info.type == EDIT_MEMBER_TYPE_REMOVE)
		list = data_group_edit_get_contacts_to_remove(s_info.group_id);

	s_info.check_list = data_group_edit_create_list_data(s_info.group_id, list);
	EINA_LIST_FOREACH(s_info.check_list, l, check_data)
		if (check_data)
			elm_genlist_item_append(s_info.genlist, itc, check_data, NULL, ELM_GENLIST_ITEM_NONE, NULL, check_data);

	elm_genlist_item_class_free(itc);
	contacts_list_destroy(list, true);
}

/**
 * @brief Internal function which displays the popup window with error message.
 */
static void _display_popup(void)
{
	const char *text;

	if (s_info.check_list)
		text = "Select member.";
	else
		text = "None member exists.";

	view_create_popup(s_info.navi, "Error", text, NULL);
}
